use wasm_bindgen::prelude::*;
use web_sys::{window, HtmlElement, MouseEvent};

#[wasm_bindgen(start)]
fn main() -> Result<(), JsValue> {
    let document = window().unwrap().document().unwrap();
    let body = document.body().unwrap();

    let cursor = document.create_element("div")?.dyn_into::<HtmlElement>()?;
    cursor.set_class_name("cursor");
    cursor.style().set_property("visibility", "hidden")?;

    body.append_child(&cursor)?;

    let mut first_move = true;

    let cursor_clone = cursor.clone();
    let event_handler =
        Closure::wrap(
            Box::new(move |event: MouseEvent| match event.type_().as_str() {
                "mousemove" => {
                    if first_move {
                        cursor_clone
                            .style()
                            .set_property("visibility", "visible")
                            .unwrap();
                        first_move = false;
                    }

                    set_cursor_position(&cursor_clone, event.client_x(), event.client_y())
                }
                "mousedown" => set_cursor_size(&cursor_clone, 18),
                "mouseup" => set_cursor_size(&cursor_clone, 12),
                "mouseout" => cursor_clone
                    .style()
                    .set_property("visibility", "hidden")
                    .unwrap(),
                "mouseover" => cursor_clone
                    .style()
                    .set_property("visibility", "visible")
                    .unwrap(),
                _ => unreachable!(),
            }) as Box<dyn FnMut(_)>,
        );

    for event in ["mousemove", "mousedown", "mouseup", "mouseout", "mouseover"] {
        document.add_event_listener_with_callback(event, event_handler.as_ref().unchecked_ref())?;
    }

    event_handler.forget();

    let elements = document.query_selector_all("a")?;
    for i in 0..elements.length() {
        let element = elements.get(i).unwrap();

        let cursor_clone = cursor.clone();
        let event_handler =
            Closure::wrap(
                Box::new(move |event: MouseEvent| match event.type_().as_str() {
                    "mouseenter" => set_cursor_size(&cursor_clone, 24),
                    "mouseleave" => set_cursor_size(&cursor_clone, 12),
                    "mousedown" => set_cursor_size(&cursor_clone, 18),
                    "mouseup" => set_cursor_size(&cursor_clone, 24),
                    _ => unreachable!(),
                }) as Box<dyn FnMut(_)>,
            );

        for event in ["mouseenter", "mouseleave", "mousedown", "mouseup"] {
            element
                .add_event_listener_with_callback(event, event_handler.as_ref().unchecked_ref())?;
        }

        event_handler.forget();
    }

    Ok(())
}

fn set_cursor_position(cursor: &HtmlElement, x: i32, y: i32) {
    cursor
        .style()
        .set_property("left", &format!("{}px", x))
        .unwrap();
    cursor
        .style()
        .set_property("top", &format!("{}px", y))
        .unwrap();
}

fn set_cursor_size(cursor: &HtmlElement, size: i32) {
    cursor
        .style()
        .set_property("width", &format!("{}px", size))
        .unwrap();
    cursor
        .style()
        .set_property("height", &format!("{}px", size))
        .unwrap();
}
